﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;

namespace mdsScript_Demo
{
    public partial class frmDemo : Form
    {
        mdsScript.ScriptingEngine se;                                                               // Scripting Engine
        string sRC;                                                                                 // Return code from script

        public frmDemo()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Select script file
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSelect_Click(object sender, EventArgs e)
        {
            string ls_File = txtScriptFile.Text.Trim();

            openFileDialog1.Title = "Open script file";
            openFileDialog1.CheckFileExists = true;
            openFileDialog1.CheckPathExists = true;
            openFileDialog1.DefaultExt = "txt";
            openFileDialog1.ValidateNames = true;
            openFileDialog1.Filter = "script file (*.txt)|*.txt|All files (*.*)|*.*";
            if (ls_File.IndexOf("\\") >= 0)
            {
                openFileDialog1.InitialDirectory = ls_File.Substring(0, ls_File.LastIndexOf("\\"));
                openFileDialog1.FileName = ls_File.Substring(ls_File.LastIndexOf("\\") + 1);
            }
            else
            {
                openFileDialog1.InitialDirectory = ls_File;
                openFileDialog1.FileName = ls_File;
            }
            DialogResult ldr_RC = openFileDialog1.ShowDialog();
            if (ldr_RC == DialogResult.OK)
            {
                ls_File = openFileDialog1.FileName.Trim();
                if (File.Exists(ls_File) == false)
                {
                    MessageBox.Show("The script file '" + ls_File + " ' does not exist!", "Open script file", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    txtScriptFile.Text = ls_File;
                }
            }
        }

        /// <summary>
        /// Execute script in the debugger window
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDebug_Click(object sender, EventArgs e)
        {
            se = new mdsScript.ScriptingEngine();                                                   // Create scripting engine instance
            se.ExternalFunctionEvent += DoExternalFunction;                                         // Set event method

            List<string> lls_GVars = new List<string>();                                            // Fill global vars string list
            //lls_GVars.Add("GVar=12345");                                                          // Example ínitializing a global var

            if (chkArray.Checked)
            {
                string[] las_Skript = ReadFile(txtScriptFile.Text.Trim());                          // Read script file to string array
                sRC = se.RunScript(las_Skript, lls_GVars, true, null);                              // Execute script
                // If no initialization of global vars is necessary please set the 2nd parameter to "null"
            }
            else
            {
                sRC = se.RunScript(txtScriptFile.Text.Trim(), lls_GVars, true, null);               // Execute script
                // If no initialization of global vars is necessary please set the 2nd parameter to "null"
            }

            List<string> lls_Data = se.GetResultData;                                               // Read result data
            string ls_Err = se.GetError;                                                            // Read error status = "sRC"

            if (ls_Err == "")                                                                       // No Error: Show result data
            {
                string ls_Temp = "The script has been finished successful.\n\nResult data:";
                for (int i = 0; i < lls_Data.Count; i++)
                {
                    ls_Temp += "\n" + lls_Data[i];
                }
                MessageBox.Show(ls_Temp, "Script finished - no errors", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else                                                                                    // Error: Show error message
            {
                MessageBox.Show("Error message:\n\n" + ls_Err, "Script finished - error occured", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Execute script and write the output to a new window 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExec_Click(object sender, EventArgs e)
        {
            se = new mdsScript.ScriptingEngine();                                                   // Create scripting engine instance
            se.ExternalFunctionEvent += DoExternalFunction;                                         // Set event method

            List<string> lls_GVars = new List<string>();                                            // Fill global vars string list
            //lls_GVars.Add("GVar=12345");                                                          // Example ínitializing a global var

            if (chkArray.Checked)
            {
                string[] las_Skript = ReadFile(txtScriptFile.Text.Trim());                          // Read script file to string array
                sRC = se.RunScript(las_Skript, lls_GVars, false, null);                             // Execute script
                // If no initialization of global vars is necessary please set the 2nd parameter to "null"
            }
            else
            {
                sRC = se.RunScript(txtScriptFile.Text.Trim(), lls_GVars, false, null);              // Execute script
                // If no initialization of global vars is necessary please set the 2nd parameter to "null"
            }

            List<string> lls_Data = se.GetResultData;                                               // Read result data
            string ls_Err = se.GetError;                                                            // Read error status = "sRC"

            if (ls_Err == "")                                                                       // No Error: Show result data
            {
                string ls_Temp = "The script has been finished successful.\n\nResult data:";
                for (int i = 0; i < lls_Data.Count; i++)
                {
                    ls_Temp += "\n" + lls_Data[i];
                }
                MessageBox.Show(ls_Temp, "Script finished - no errors", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else                                                                                    // Error: Show error message
            {
                MessageBox.Show("Error message:\n\n" + ls_Err, "Script finished - error occured", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Execute script and write the output lines in the list box 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnExecInWindow_Click_1(object sender, EventArgs e)
        {
            se = new mdsScript.ScriptingEngine();                                                   // Create scripting engine instance
            se.ExternalFunctionEvent += DoExternalFunction;                                         // Set event method

            List<string> lls_GVars = new List<string>();                                            // Fill global vars string list
            //lls_GVars.Add("GVar=12345");                                                          // Example ínitializing a global var

            if (chkArray.Checked)
            {
                string[] las_Skript = ReadFile(txtScriptFile.Text.Trim());                          // Read script file to string array
                sRC = se.RunScript(las_Skript, lls_GVars, false, lstOut);                           // Execute script
                // If no initialization of global vars is necessary please set the 2nd parameter to "null"
            }
            else
            {
                sRC = se.RunScript(txtScriptFile.Text.Trim(), lls_GVars, false, lstOut);            // Execute script
                // If no initialization of global vars is necessary please set the 2nd parameter to "null"
            }

            List<string> lls_Data = se.GetResultData;                                               // Read result data
            string ls_Err = se.GetError;                                                            // Read error status = "sRC"

            if (ls_Err == "")                                                                       // No Error: Show result data
            {
                string ls_Temp = "The script has been finished successful.\n\nResult data:";
                for (int i = 0; i < lls_Data.Count; i++)
                {
                    ls_Temp += "\n" + lls_Data[i];
                }
                MessageBox.Show(ls_Temp, "Script finished - no errors", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else                                                                                    // Error: Show error message
            {
                MessageBox.Show("Error message:\n\n" + ls_Err, "Script finished - error occured", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        

        /// <summary>
        /// Read a text file and encode the text with the default codepage
        /// </summary>
        /// <param name="ps_Filename">File name</param>
        /// <returns>String array containing file data</returns>
        private string[] ReadFile(string ps_Filename)
        {
            List<string> lls_Data = new List<string>();

            StreamReader srFile = new StreamReader(ps_Filename, System.Text.Encoding.Default);
            while (srFile.EndOfStream == false)
            {
                lls_Data.Add(srFile.ReadLine());
            }
            srFile.Close();

            string[] las_Data = new string[lls_Data.Count];
            for (int i = 0; i < lls_Data.Count; i++)
                las_Data[i] = lls_Data[i];

            return las_Data;
        }

        /// <summary>
        /// External functions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DoExternalFunction(object sender, mdsScript.ExternalFunctionEventArgs e)
        {
            // Function name:       e.FunctionName
            // Parameter list:      e.ParmsList

            string ls_Temp;
            switch (e.FunctionName.ToUpper().Trim())
            {
                // The TEST.* methods/functions show how to communicate with the scripting engine: 

                case "TEST.PROC":
                    ls_Temp = "External procedure called: " + e.FunctionName + " - Parameter:";
                    for (int i = 0; i < e.ParmsList.Count; i++)
                    {
                        ls_Temp += "    " + e.ParmsList[i];
                    }
                    se.ExtFunc_WriteOutputText("TEST.PROC: " + ls_Temp, true);                      // Write text to the script output data
                    break;
                case "TEST.FUNC":
                    ls_Temp = "External function called: " + e.FunctionName + " - Parameter:";
                    for (int i = 0; i < e.ParmsList.Count; i++)
                    {
                        ls_Temp += "    " + e.ParmsList[i];
                    }
                    se.ExtFunc_WriteResultText("TEST.FUNC: RetCode=" + e.ParmsList[0]);             // Write return value in scipt result data
                    se.ExtFunc_SetReturnCode("TEST.FUNC: RetCode=" + e.ParmsList[0], "");           // Set function value
                    break;
                case "TEST.SET_GLOBAL_VAR":
                    if (e.ParmsList.Count < 2)
                        se.ExtFunc_SetReturnCode("", "Function parameter count invalid!");          // Set error value
                    else
                    {
                        bool lb_Success = se.ExtFunc_SetGlobalVar(e.ParmsList[0], e.ParmsList[1]);  // Write global variable
                        if (lb_Success)
                            se.ExtFunc_WriteOutputText("The global variable '" + e.ParmsList[0] + "' was changed to '" + e.ParmsList[1] + "'.", false);
                        else
                            se.ExtFunc_WriteOutputText("The global variable '" + e.ParmsList[0] + "' does not exist!", false);
                    }
                    break;
                case "TEST.GET_GLOBAL_VAR":
                    if (e.ParmsList.Count < 1)
                        se.ExtFunc_SetReturnCode("", "Function parameter count invalid!");          // Set error value
                    else
                    {
                        bool lb_Exists;
                        ls_Temp = se.ExtFunc_GetGlobalVar(e.ParmsList[0], out lb_Exists);           // Read global variable
                        if (lb_Exists)
                        {
                            se.ExtFunc_WriteResultText("The value of the global variable '" + e.ParmsList[0] + "' is '" + ls_Temp + "'");
                            se.ExtFunc_SetReturnCode(ls_Temp, "");                                  // Set function value
                        }
                        else
                        {
                            se.ExtFunc_WriteResultText("The global variable '" + e.ParmsList[0] + "' does not exist!");
                            se.ExtFunc_SetReturnCode("", "Variable not found!");                    // Set error value
                        }
                    }
                    break;
                case "TEST.WRITE_FILE":
                    if (e.ParmsList.Count < 1)
                        se.ExtFunc_SetReturnCode("", "Function parameter count invalid!");          // Set error value
                    else
                        se.ExtFunc_WriteFileText(int.Parse(e.ParmsList[0]), e.ParmsList[1], true);
                    break;
                default:
                    se.ExtFunc_SetReturnCode("", "Function name invalid: " + e.FunctionName.ToUpper().Trim()); // Set error value
                    break;
            }
        }
    }
}
